/*  Canvas Widget
 *  Copyleft (C) 2001 David Griffiths <dave@pawfal.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/ 

#include "Fl/fl_draw.H"
#include "Fl_Canvas.h"
#include "Fl_DeviceGUI.h"
#include <iostream.h>
#include "../../SpiralSynthModularInfo.h"

#define IMG_BACKGROUND

#ifdef IMG_BACKGROUND
#include <png.h>
#endif

Fl_Canvas::Fl_Canvas(int x, int y, int w, int h, char *name) :
Fl_Group(x,y,w,h,name),
m_Status(NULL),
cb_Connection(NULL),
cb_Unconnect(NULL)
{
	m_IncompleteWire.OutputChild=-1;
	m_IncompleteWire.OutputPort=-1;
	m_IncompleteWire.InputChild=-1;
	m_IncompleteWire.InputPort=-1;
	
	m_BG=NULL;
	m_BGData=NULL;
	
#ifdef IMG_BACKGROUND
	if (SpiralSynthModularInfo::BGIMG=="None") return;
	FILE *fp=fopen(SpiralSynthModularInfo::BGIMG.c_str(),"rb");
	if (!fp) cerr<<"Couldn't open image ["<<SpiralSynthModularInfo::BGIMG.c_str()<<"]"<<endl;
	else
	{
		png_structp png_ptr;
		png_infop info_ptr;
		
		png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
   	 	info_ptr = png_create_info_struct(png_ptr);
   		png_init_io(png_ptr, fp);
		png_read_png(png_ptr, info_ptr, PNG_TRANSFORM_IDENTITY, NULL);		
		fclose(fp);
		
		m_BGData = new char[png_ptr->rowbytes*png_ptr->height];
		int p=0;
		for (unsigned int row = 0; row < png_ptr->height; row++)
		{
			for (unsigned int i=0; i<png_ptr->rowbytes; i++)
			{
				m_BGData[p]=(char)info_ptr->row_pointers[row][i];
				p++;
			}
		}
		m_BG = new Fl_Image((unsigned char*)m_BGData,(int)png_ptr->width,(int)png_ptr->height);
		png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp)NULL);	
	}
#endif 
	
}

Fl_Canvas::~Fl_Canvas()
{
#ifdef IMG_BACKGROUND
	if (m_BGData) delete[] m_BGData;
#endif 	
}

void Fl_Canvas::draw()
{	
	clear_damage(FL_DAMAGE_ALL);	
	fl_push_no_clip();
	fl_push_clip(parent()->x(),parent()->y(),parent()->w()-16,parent()->h()-16);
	

	//Fl_Group::draw();
	
	Fl_Widget*const* a = array();
  	if (damage() & ~FL_DAMAGE_CHILD) { // redraw the entire thing:
    
	if (m_BG)
	{
		int X=0,Y=0;
		while (Y<w())
		{	
			while (X<h())
			{
				m_BG->draw(parent()->x()+X,parent()->y()+Y);
				X+=m_BG->w;
			}		
			Y+=m_BG->h;
			X=0;
		}
	}
	else
	{
		draw_box();
	}
			
    for (int i=children(); i--;) {
      Fl_Widget& o = **a++;
      draw_child(o);
      draw_outside_label(o);
    }
	DrawWires();
  } else {	// only redraw the children that need it:
    for (int i=children(); i--;) update_child(**a++);
  }

	fl_pop_clip();
	fl_pop_clip();
}

void Fl_Canvas::DrawWires()
{
	for(vector<CanvasWire>::iterator i=m_WireVec.begin();
		i!=m_WireVec.end(); i++)
	{
		if (i->OutputChild>children() || i->InputChild>children())
		{
			cerr<<"wire output child = "<<i->OutputChild<<endl;
			cerr<<"wire input child = "<<i->InputChild<<endl;
			assert(0);
		}
			   
		Fl_DeviceGUI* SourceDevice = (Fl_DeviceGUI*)(child(i->OutputChild));
		Fl_DeviceGUI* DestDevice   = (Fl_DeviceGUI*)(child(i->InputChild));
			 
		assert(SourceDevice && DestDevice);
					 
		fl_color(FL_RED);
		fl_line(SourceDevice->GetPortX(i->OutputPort+SourceDevice->GetInfo()->NumInputs),
				SourceDevice->GetPortY(i->OutputPort+SourceDevice->GetInfo()->NumInputs),
				DestDevice->GetPortX(i->InputPort),
				DestDevice->GetPortY(i->InputPort));	
	}
}
		
int Fl_Canvas::handle(int event)
{
	return Fl_Group::handle(event);
}
	
void Fl_Canvas::PortClicked(Fl_DeviceGUI* Device, int Type, int Port, bool Value)
{
	// find out which child this comes from.
	int ChildNum=-1;	
	for(int n=0; n<children(); n++)
	{
		if(child(n)==Device)
		{
			ChildNum=n;
		}
	}
	
	assert(ChildNum!=-1);
	
	if(Value) // Turned on the port
	{
		if(m_IncompleteWire.InputChild==-1 || m_IncompleteWire.OutputChild==-1)
		{
			if (Type==Fl_DeviceGUI::OUTPUT)
			{
				// make sure we don't make a output->output connection
				if (m_IncompleteWire.OutputChild==-1)
				{	
					m_IncompleteWire.OutputChild=ChildNum;
					m_IncompleteWire.OutputPort=Port;
					m_IncompleteWire.OutputID=Device->GetID();
				}
				else
				{
					m_IncompleteWire.Clear();
				}
			}
			else
			{
				// make sure we don't make a input->input connection
				if (m_IncompleteWire.InputChild==-1)
				{	
					m_IncompleteWire.InputChild=ChildNum;
					m_IncompleteWire.InputPort=Port;
					m_IncompleteWire.InputID=Device->GetID();
				}
				else
				{
					m_IncompleteWire.Clear();
				}
			}

			// if both have now been set...
			if (m_IncompleteWire.InputChild!=-1 && m_IncompleteWire.OutputChild!=-1)
			{
				m_WireVec.push_back(m_IncompleteWire);	
	
				// send the connect callback
				cb_Connection(this,(void*)&m_IncompleteWire);
				
				m_IncompleteWire.Clear();
				
				redraw();
			}
		}
	}
	else // Turned off the port
	{	
		// Find the connection
		for(vector<CanvasWire>::iterator i=m_WireVec.begin();
			i!=m_WireVec.end(); i++)
		{
			if ((Type==Fl_DeviceGUI::OUTPUT && i->OutputChild==ChildNum && i->OutputPort==Port) ||
				(Type==Fl_DeviceGUI::INPUT && i->InputChild==ChildNum && i->InputPort==Port))
			{
				// Turn off both ports
				((Fl_DeviceGUI*)(child(i->OutputChild)))->ForcePortValue(i->OutputPort+
					((Fl_DeviceGUI*)(child(i->OutputChild)))->GetInfo()->NumInputs,false);
				((Fl_DeviceGUI*)(child(i->InputChild)))->ForcePortValue(i->InputPort,false);
				
				// send the unconnect callback	
				cb_Unconnect(this,(void*)&(*i));
				
				// Remove the wire
				m_WireVec.erase(i);
				
				redraw();
				break;
			}
		}
		
		// Clear the current selection
		m_IncompleteWire.Clear();
	}
}

void Fl_Canvas::ClearConnections(Fl_DeviceGUI* Device)
{
	// find out which child this comes from.
	int ChildNum=-1;	
	for(int n=0; n<children(); n++)
	{
		if(child(n)==Device)
		{
			ChildNum=n;
		}
	}
	
	assert(ChildNum!=-1);
	
	bool removedall=false;
		
	while (!removedall)
	{
		removedall=true;
		
		for (vector<CanvasWire>::iterator i=m_WireVec.begin();
			 i!=m_WireVec.end(); i++)
		{	
			if (i->OutputChild==ChildNum ||
			    i->InputChild==ChildNum)
			{
				// Turn off both ports
				((Fl_DeviceGUI*)(child(i->OutputChild)))->ForcePortValue(i->OutputPort+
					((Fl_DeviceGUI*)(child(i->OutputChild)))->GetInfo()->NumInputs,false);
				((Fl_DeviceGUI*)(child(i->InputChild)))->ForcePortValue(i->InputPort,false);

				// send the unconnect callback
				cb_Unconnect(this,(void*)&(*i));
				
				m_WireVec.erase(i);
				removedall=false;
				break;
			}
		}
	}
}
	
void Fl_Canvas::RemoveDevice(Fl_DeviceGUI* Device)
{
	// find out which child this comes from.
	int ChildNum=-1;	
	for(int n=0; n<children(); n++)
	{
		if(child(n)==Device)
		{
			ChildNum=n;
		}
	}
	
	assert(ChildNum!=-1);
	ClearConnections(Device);

	for (vector<CanvasWire>::iterator i=m_WireVec.begin();
		 i!=m_WireVec.end(); i++)
	{	
		if (i->OutputChild>ChildNum) i->OutputChild--;		
		if (i->InputChild>ChildNum)	 i->InputChild--;
	}
	
	remove(child(ChildNum));
	redraw(); 
}
	
void Fl_Canvas::Clear() 
{ 
	int kids=children();

	for(int n=0; n<kids; n++)
	{
		remove(child(0));		
	}	
	
	m_WireVec.clear(); 
	redraw(); 
}

	
	
istream &operator>>(istream &s, Fl_Canvas &o)
{
	int NumWires;
	s>>NumWires;
		
	for(int n=0; n<NumWires; n++)
	{
		CanvasWire NewWire;
		
		s>>NewWire.OutputID;
		s>>NewWire.OutputChild;
		s>>NewWire.OutputPort;
		s>>NewWire.InputID;
		s>>NewWire.InputChild;
		s>>NewWire.InputPort;	

		o.m_WireVec.push_back(NewWire);

		// Notify connection by callback
		o.cb_Connection(&o,(void*)&NewWire);
	
		// Turn on both ports
		((Fl_DeviceGUI*)(o.child(NewWire.OutputChild)))->ForcePortValue(NewWire.OutputPort+
			((Fl_DeviceGUI*)(o.child(NewWire.OutputChild)))->GetInfo()->NumInputs,true);
		((Fl_DeviceGUI*)(o.child(NewWire.InputChild)))->ForcePortValue(NewWire.InputPort,true);
	}
		
	return s;
}

ostream &operator<<(ostream &s, Fl_Canvas &o)
{
	s<<o.m_WireVec.size()<<endl;

	for(vector<CanvasWire>::iterator i=o.m_WireVec.begin();
		i!=o.m_WireVec.end(); i++)
	{
		s<<i->OutputID<<" ";
		s<<i->OutputChild<<" ";
		s<<i->OutputPort<<" ";
		s<<i->InputID<<" ";
		s<<i->InputChild<<" ";
		s<<i->InputPort<<endl;	
	}

	return s;
}
