/*  DeviceGUI composite Widget
 *  Copyleft (C) 2001 David Griffiths <dave@pawfal.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/ 

#include "Fl_DeviceGUI.h"
#include "Fl_Canvas.h"
#include "../../SpiralSynthModularInfo.h"

int Fl_DeviceGUI::Numbers[512];

Fl_DeviceGUI::Fl_DeviceGUI(const DeviceGUIInfo& Info, Fl_Window *PW, Fl_Pixmap *Icon) :
Fl_Group(Info.XPos, Info.YPos, Info.Width+(PortGroupWidth*2), Info.Height+TitleBarHeight, ""),
m_PluginWindow(NULL),
m_Name(Info.Name),
m_ID(-1),
m_DelMe(false),
m_WindowShown(false)
{	
	for (int n=0; n<512; n++) Numbers[n]=n;
	
	type(1);
	box(FL_THIN_UP_BOX);
    labeltype(FL_ENGRAVED_LABEL);
    align(FL_ALIGN_TOP_LEFT|FL_ALIGN_INSIDE);
	color(SpiralSynthModularInfo::GUICOL_Device);
	
	m_DragBar = new Fl_DragBar(Info.XPos, Info.YPos, Info.Width+PortGroupWidth*2, TitleBarHeight, m_Name.c_str());
	m_DragBar->labelsize(10);
	m_DragBar->color(SpiralSynthModularInfo::GUICOL_Device);
	
	m_PluginButton = new Fl_Button(Info.XPos+PortGroupWidth,Info.YPos+TitleBarHeight,Info.Width,Info.Height-TitleBarHeight,m_Name.c_str());
	if (Icon) Icon->label(m_PluginButton);   	 	
	m_PluginButton->box(FL_NO_BOX);
	m_PluginButton->color(SpiralSynthModularInfo::GUICOL_Device);
	m_PluginButton->selection_color(SpiralSynthModularInfo::GUICOL_Device);
	m_PluginButton->callback((Fl_Callback*)cb_Plugin);		
	
	m_PluginWindow = PW;
	
	//Add the input/output ports
	Setup(Info, true);
}

void Fl_DeviceGUI::Clear()
{
	/*for (vector<Fl_Round_Button*>::iterator i = m_PortVec.begin();
		 i!=m_PortVec.end(); i++)
	{
		remove(*i);
		delete *i;
	}
	m_PortVec.clear();
	
	remove(m_PluginButton);
	delete m_PluginButton;
	remove(m_DragBar);
	delete m_DragBar;
	*/
	end();
}

int  Fl_DeviceGUI::handle(int event)
{
	if (m_PortTips.size()>0)
	{
		//Check ports for mouseover
		unsigned int n=0;
		for(vector<Fl_Round_Button*>::iterator i=m_PortVec.begin();
			i!=m_PortVec.end(); i++)
		{
			if (Fl::belowmouse()==*i && n<m_PortTips.size())
			{
				((Fl_Canvas*)(parent()))->DisplayText(m_PortTips[n]);
			}
			n++;
		}
	}
	
	if (Fl::event_button()==2) 
	{		
		m_DelMe = true;
	}
	
	return Fl_Group::handle(event);
}

void Fl_DeviceGUI::Setup(const DeviceGUIInfo& Info, bool FirstTime)
{
	m_Info=Info;
	
	// Remove all current connections - it's the safest thing to do.
	if (parent() && !FirstTime) 
	{
		((Fl_Canvas*)(parent()))->ClearConnections(this);
	}

	// delete the current ports
	for(vector<Fl_Round_Button*>::iterator i=m_PortVec.begin();
			i!=m_PortVec.end(); i++)
	{		
		remove(*i);
		delete(*i);
	}

	m_PortVec.clear();
	m_PortTips.clear();
	
	int InputX=x()+2;
	int OutputX=x()+PortGroupWidth+Info.Width+2;
	int StartY=y()+TitleBarHeight;
	int PortDist=10;
	int PortNum=0;
	
	h(Info.Height+TitleBarHeight);
	m_PluginButton->size(m_PluginButton->w(),Info.Height-TitleBarHeight);
	
	for (int n=0; n<Info.NumInputs; n++)
	{
		Fl_Round_Button* NewInput = new Fl_Round_Button(InputX,StartY+PortDist*n,PortSize,PortSize,"");
		NewInput->value(false);
		NewInput->down_box(FL_ROUNDED_BOX);
		NewInput->callback((Fl_Callback*)cb_Port,(void*)&Numbers[PortNum]);		
		m_PortVec.push_back(NewInput);
		add(NewInput);
		PortNum++;
	}
	
	for (int n=0; n<Info.NumOutputs; n++)
	{
		Fl_Round_Button* NewOutput = new Fl_Round_Button(OutputX,StartY+PortDist*n,PortSize,PortSize,"");
		NewOutput->value(false);
		NewOutput->down_box(FL_ROUNDED_BOX);
		NewOutput->callback((Fl_Callback*)cb_Port,(void*)&Numbers[PortNum]);		
		m_PortVec.push_back(NewOutput);
		add(NewOutput);
		PortNum++;
	}
	
	// Copy the porttip text
	m_PortTips=Info.PortTips;
}

inline void Fl_DeviceGUI::cb_Port_i(Fl_Round_Button* o, void* v)
{
	int Port=*(int*)(v);
	PortType Pt;
	
	if (m_DelMe) return;
	
	// Find out if this is an input or an output.
	if (Port<m_Info.NumInputs) 
	{
		Pt=INPUT;
	}
	else
	{
		Pt=OUTPUT;
		Port-=m_Info.NumInputs;
	}
			
	((Fl_Canvas*)(parent()))->PortClicked(this,Pt,Port,o->value());
		
		
}
void Fl_DeviceGUI::cb_Port(Fl_Round_Button* o, void* v)
{((Fl_DeviceGUI*)(o->parent()))->cb_Port_i(o,v);}

inline void Fl_DeviceGUI::cb_Plugin_i(Fl_Button* o, void* v)
{
	if (m_PluginWindow && !m_DelMe)
	{
		m_WindowShown=!m_WindowShown;		
		if (m_WindowShown) m_PluginWindow->show();
		else m_PluginWindow->hide();
	}
}
void Fl_DeviceGUI::cb_Plugin(Fl_Button* o, void* v)
{((Fl_DeviceGUI*)(o->parent()))->cb_Plugin_i(o,v);}
