/*  DeviceGUI composite Widget
 *  Copyleft (C) 2001 David Griffiths <dave@pawfal.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/ 

#include <FL/Fl.H>
#include <FL/Fl_Group.H>
#include <FL/Fl_Round_Button.H>
#include <FL/Fl_Pixmap.h>
#include <iostream.h>
#include <vector>
#include <string>
#include "Fl_DragBar.H"

#ifndef DEVICEGUI
#define DEVICEGUI

static const int TitleBarHeight = 12;
static const int PortGroupWidth = 12;
static const int PortSize=10;

struct DeviceGUIInfo
{
	int XPos;
	int YPos;	
	int Width;
	int Height;
	int NumInputs;
	int NumOutputs;
	vector<string> PortTips;
	string Name;
};

class Fl_DeviceGUI : public Fl_Group
{
public:
	Fl_DeviceGUI(const DeviceGUIInfo& Info, Fl_Window *PW, Fl_Pixmap *Icon);

	virtual int handle(int event);
	
	enum PortType {INPUT,OUTPUT};
		
	int  GetID()           			 { return m_ID; }
	void SetID(int s)      			 { m_ID=s; }
	bool Killed()          			 { return m_DelMe; }
	
	int   GetPortX(int n) { return m_PortVec[n]->x()+PortSize/2; }
	int   GetPortY(int n) { return m_PortVec[n]->y()+PortSize/2; }
	void  ForcePortValue(int n, bool v) { m_PortVec[n]->value(v); }	
	const DeviceGUIInfo* GetInfo() { return &m_Info; }
	// automatically called from the constructor, but may be redone at any time.
	virtual void  Setup(const DeviceGUIInfo& Info, bool FirstTime = false); 
	virtual void  Clear();
	
protected:

	DeviceGUIInfo m_Info;

	Fl_DragBar* m_DragBar;
	Fl_Button*  m_PluginButton;
	Fl_Window*  m_PluginWindow;
		
private:

	inline void cb_Port_i(Fl_Round_Button* o, void* v);
	static void cb_Port(Fl_Round_Button* o, void* v);
	inline void cb_Plugin_i(Fl_Button* o, void* v);
	static void cb_Plugin(Fl_Button* o, void* v);

	vector<Fl_Round_Button*> m_PortVec;
	vector<string> m_PortTips;

	static int Numbers[512];

	string m_Name;
	int    m_ID;
	bool   m_DelMe;
	bool   m_WindowShown;
};

#endif
