/*  SpiralSound
 *  Copyleft (C) 2001 David Griffiths <dave@pawfal.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/ 

#include <dlfcn.h>
#include <stdio.h>
#include "PluginManager.h"

PluginManager *PluginManager::m_Singleton = NULL;

PluginManager::PluginManager()
{
}
 
PluginManager::~PluginManager()
{
	UnloadAll();
}
	
PluginID PluginManager::LoadPlugin(const char *PluginName)
{
	// Make a new plugin
	HostsideInfo *NewPlugin = new HostsideInfo;
	NewPlugin->Handle=NULL;
	NewPlugin->ID=PluginError;
	
	//cerr<<"Opening "<<PluginName<<endl;
	
	// Attempt to open the plugin
	NewPlugin->Handle = dlopen (PluginName, RTLD_NOW);
        
    if (NewPlugin->Handle==NULL)
    {
      	cerr<<"Error loading plugin: "<<endl;
        fputs(dlerror(), stderr);
        return PluginError;
    }
		
	// Link the neccesary functions
	NewPlugin->CreateInstance  = (SpiralPlugin*(*)()) dlsym(NewPlugin->Handle, "CreateInstance__Fv"); 	
	NewPlugin->GetIcon = (Fl_Pixmap*(*)()) dlsym(NewPlugin->Handle, "GetIcon__Fv");
	NewPlugin->GetID  = (int(*)()) dlsym(NewPlugin->Handle, "GetID__Fv"); 	
	
	char *error;		
    if ((error = dlerror()) != NULL)
    {
    	cerr<<"Error linking to plugin:"<<endl;
	    fputs(error, stderr);
		cerr<<endl;
        return PluginError;
    }       
		
	// We've succesfully open and linked the
	// plugin, so add it to the vector.
	int ID;
	ID = NewPlugin->GetID();
	NewPlugin->ID = ID;
	m_PluginVec.push_back(NewPlugin);
	
    return ID;
}

void PluginManager::UnLoadPlugin(PluginID ID)
{
	if (IsValid(ID))
    {
		dlclose(GetPlugin(ID)->Handle);
		char *error;				
	    if ((error = dlerror()) != NULL)
	    {
    		cerr<<"Error unlinking plugin: "<<endl;
		    fputs(error, stderr);
			cerr<<endl;
   		}       
	}
}

void PluginManager::UnloadAll()
{
	for (vector<HostsideInfo*>::iterator i=m_PluginVec.begin();
		 i!=m_PluginVec.end(); i++)
	{		
		dlclose((*i)->Handle);
		char *error;				
	    if ((error = dlerror()) != NULL)
	    {
    		cerr<<"Error unlinking plugin: "<<endl;
		    fputs(error, stderr);
			cerr<<endl;
   		}       
	}
}

const HostsideInfo *PluginManager::GetPlugin(PluginID ID)
{
	return GetPlugin_i(ID);
}

HostsideInfo *PluginManager::GetPlugin_i(PluginID ID)
{

	HostsideInfo *ret = NULL;

	for (vector<HostsideInfo*>::iterator i=m_PluginVec.begin();
		 i!=m_PluginVec.end(); i++)
	{
		if ((*i)->ID==ID)
		{
			ret = *i;
			break;
		}
	}

	if (!ret)
	{
		cerr<<"Warning: Plugin "<<ID<<" not found."<<endl;
	}

	return ret;
}
	
bool PluginManager::IsValid(PluginID ID) 
{ 
	const HostsideInfo *t = GetPlugin(ID);
	return (t && t->Handle!=NULL); 
}


