/*  SpiralSound
 *  Copyleft (C) 2001 David Griffiths <dave@pawfal.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/ 
#include "DelayPlugin.h"
#include "DelayPluginGUI.h"
#include <FL/Fl_Button.h>
#include "SpiralIcon.xpm"

static const float MAX_DELAY=1.0f;

SpiralPlugin* CreateInstance()
{
	return new DelayPlugin;
}

Fl_Pixmap* GetIcon()
{
	return new Fl_Pixmap(SpiralIcon_xpm);
}

int GetID()
{
	return 0x000f;
}

///////////////////////////////////////////////////////

DelayPlugin::DelayPlugin() :
m_Delay(0.75),
m_Mix(0.4),
m_ReadHeadPos(0),
m_WriteHeadPos(0)
{
	m_PluginInfo.Name="Delay";
	m_PluginInfo.Width=120;
	m_PluginInfo.Height=110;
	m_PluginInfo.NumInputs=3;
	m_PluginInfo.NumOutputs=1;
	m_PluginInfo.PortTips.push_back("Input");	
	m_PluginInfo.PortTips.push_back("Delay CV");		
	m_PluginInfo.PortTips.push_back("ReadHead CV");	
	m_PluginInfo.PortTips.push_back("Output");
}

DelayPlugin::~DelayPlugin()
{
}

PluginInfo &DelayPlugin::Initialise(const HostInfo *Host)
{	
	PluginInfo& Info = SpiralPlugin::Initialise(Host);
	Sample *NewSample = new Sample(Host->BUFSIZE);
	m_Output[0]=NewSample;
	
	m_Buffer.Allocate(m_HostInfo->SAMPLERATE*MAX_DELAY);
	
	return Info;
}

Fl_Window *DelayPlugin::CreateGUI()
{
	m_GUI = new DelayPluginGUI(m_PluginInfo.Width,
								  	    m_PluginInfo.Height,
										this,m_HostInfo);
	m_GUI->hide();
	return m_GUI;
}

void DelayPlugin::Execute()
{
 	Fl::check();
	
	int Delay;
	
    for (int n=0; n<m_HostInfo->BUFSIZE; n++)
	{
		Delay=(int)((m_Delay+GetInputCV(1,n)*0.5)*m_HostInfo->SAMPLERATE);
		
		// Read from the buffer
		m_Output[0]->Set(n,m_Buffer[(int)m_ReadHeadPos]+m_Mix*GetInputVal(0,n));	 
		
		// Write to the buffer	
		m_Buffer.Set((int)m_WriteHeadPos,GetInputVal(0,n));
		
		m_WriteHeadPos++;	
		m_ReadHeadPos=m_WriteHeadPos+GetInputCV(2,n)*Delay;
		
		if (m_ReadHeadPos<0) m_ReadHeadPos=Delay+m_ReadHeadPos;
		if (m_WriteHeadPos<0) m_WriteHeadPos=Delay+m_WriteHeadPos;
		
		if (m_Delay>0) 
		{	
			if (m_ReadHeadPos>=Delay) m_ReadHeadPos=m_ReadHeadPos-Delay;
			if (m_WriteHeadPos>=Delay) m_WriteHeadPos=m_WriteHeadPos-Delay;	
		}
		else
		{
			m_ReadHeadPos=0;
			m_WriteHeadPos=0;
		}
	}
		
}
	
void DelayPlugin::Randomise()
{
}
	
void DelayPlugin::StreamOut(ostream &s)
{
	s<<m_Version<<" "<<m_Delay<<" "<<m_Mix<<" ";
}

void DelayPlugin::StreamIn(istream &s)
{	
	int version;
	s>>version;
	s>>m_Delay>>m_Mix;
}

int main()
{
	Fl::visual(FL_DOUBLE|FL_RGB);
	
	HostInfo host;
	host.BUFSIZE=256;
	host.SAMPLERATE=44100;
	host.OUTPUTFILE="/dev/dsp";
	host.MIDIFILE="/dev/midi";
	host.POLY=1;
	host.GUI_COLOUR=100;
	
	SpiralPlugin* test = CreateInstance();
	test->Initialise(&host);
    test->CreateGUI();
		
    for (;;) 
	{
    	if (!Fl::check()) break;
		test->Execute();    	
  	}
	
	delete test;	
	
	return 1;	
}
