/*  SpiralSound
 *  Copyleft (C) 2001 David Griffiths <dave@pawfal.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/ 
#include "EnvFollowerPlugin.h"
#include "EnvFollowerPluginGUI.h"
#include <FL/Fl_Button.h>
#include "SpiralIcon.xpm"

SpiralPlugin* CreateInstance()
{
	return new EnvFollowerPlugin;
}

Fl_Pixmap* GetIcon()
{
	return new Fl_Pixmap(SpiralIcon_xpm);
}

int GetID()
{
	return 0x0013;
}

///////////////////////////////////////////////////////

EnvFollowerPlugin::EnvFollowerPlugin() :
m_Attack(0.5),
m_Decay(0.5),
m_Value(0)
{
	m_PluginInfo.Name="EnvFollower";
	m_PluginInfo.Width=140;
	m_PluginInfo.Height=100;
	m_PluginInfo.NumInputs=1;
	m_PluginInfo.NumOutputs=1;
	m_PluginInfo.PortTips.push_back("Input");	
	m_PluginInfo.PortTips.push_back("Output");
}

EnvFollowerPlugin::~EnvFollowerPlugin()
{
}

PluginInfo &EnvFollowerPlugin::Initialise(const HostInfo *Host)
{	
	PluginInfo& Info = SpiralPlugin::Initialise(Host);
	Sample *NewSample = new Sample(Host->BUFSIZE);
	m_Output[0]=NewSample;	
	return Info;
}

Fl_Window *EnvFollowerPlugin::CreateGUI()
{
	m_GUI = new EnvFollowerPluginGUI(m_PluginInfo.Width,
								  	    m_PluginInfo.Height,
										this,m_HostInfo);
	m_GUI->hide();
	return m_GUI;
}

void EnvFollowerPlugin::Execute()
{
	float Value;
	for (int n=0; n<m_HostInfo->BUFSIZE; n++)
	{
		Value = abs(GetInputVal(0,n));
		
		if (Value>m_Value) m_Value+=(m_Attack*100);
		else m_Value-=(m_Decay*100);
		
		m_Output[0]->Set(n,m_Value);
	}
	
 	Fl::check();
}

void EnvFollowerPlugin::StreamOut(ostream &s) 
{
	s<<m_Version<<" ";
	s<<m_Attack<<" "<<m_Decay;
}

void EnvFollowerPlugin::StreamIn(istream &s)
{
	s>>m_Version;
	s>>m_Attack>>m_Decay;
}

 
int main()
{
	Fl::visual(FL_DOUBLE|FL_RGB);
	
	HostInfo host;
	host.BUFSIZE=256;
	host.SAMPLERATE=44100;
	host.OUTPUTFILE="/dev/dsp";
	host.MIDIFILE="/dev/midi";
	host.POLY=1;
	host.GUI_COLOUR=100;
	
	
	SpiralPlugin* test = CreateInstance();
	test->Initialise(&host);
    test->CreateGUI();
		
    for (;;) 
	{
    	if (!Fl::check()) break;
		test->Execute();    	
  	}
	
	delete test;	
	
	return 1;	
}
