/*  SpiralSound
 *  Copyleft (C) 2001 David Griffiths <dave@pawfal.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
#include "MatrixPlugin.h"
#include "MatrixPluginGUI.h"
#include <FL/Fl_Button.h>
#include "SpiralIcon.xpm"
#include "../../RiffWav.h"
#include "../../NoteTable.h"

static const float TRIG_THRESH = NoteTable[0];

SpiralPlugin* CreateInstance()
{
	return new MatrixPlugin;
}

Fl_Pixmap* GetIcon()
{
	return new Fl_Pixmap(SpiralIcon_xpm);
}

int GetID()
{
	return 0x0012;
}

///////////////////////////////////////////////////////

MatrixPlugin::MatrixPlugin() :
m_TickTime(1.0f),
m_StepTime(1.0f),
m_Time(0.0f),
m_Step(0),
m_Loop(true),
m_NoteCut(false),
m_Current(0),
m_CurrentNoteCV(0),
m_CurrentTriggerCV(0),
m_Triggered(false)
{
	m_PluginInfo.Name="Matrix";
	m_PluginInfo.Width=540;
	m_PluginInfo.Height=290;
	m_PluginInfo.NumInputs=4;
	m_PluginInfo.NumOutputs=18;		
	m_PluginInfo.PortTips.push_back("Play Trigger");	
	m_PluginInfo.PortTips.push_back("StepTime CV");	
	m_PluginInfo.PortTips.push_back("Input Pitch CV");	
	m_PluginInfo.PortTips.push_back("Input Trigger CV");	
	m_PluginInfo.PortTips.push_back("Output Pitch");	
	m_PluginInfo.PortTips.push_back("Output Trigger");
	m_PluginInfo.PortTips.push_back("Trigger 1");
	m_PluginInfo.PortTips.push_back("Trigger 2");
	m_PluginInfo.PortTips.push_back("Trigger 3");
	m_PluginInfo.PortTips.push_back("Trigger 4");
	m_PluginInfo.PortTips.push_back("Trigger 5");
	m_PluginInfo.PortTips.push_back("Trigger 6");
	m_PluginInfo.PortTips.push_back("Trigger 7");
	m_PluginInfo.PortTips.push_back("Trigger 8");
	m_PluginInfo.PortTips.push_back("Trigger 9");
	m_PluginInfo.PortTips.push_back("Trigger 10");
	m_PluginInfo.PortTips.push_back("Trigger 11");
	m_PluginInfo.PortTips.push_back("Trigger 12");
	m_PluginInfo.PortTips.push_back("Trigger 13");
	m_PluginInfo.PortTips.push_back("Trigger 14");
	m_PluginInfo.PortTips.push_back("Trigger 15");
	m_PluginInfo.PortTips.push_back("Trigger 16");
			
	for (int n=0; n<NUM_PATTERNS; n++)
	{
		m_Matrix[n].Length=32;
		m_Matrix[n].Speed=1.0f;
		m_Matrix[n].Octave=0;
		
		for (int x=0; x<MATX; x++)
		for (int y=0; y<MATY; y++)
		{
			m_Matrix[n].Matrix[x][y]=false;
		}
		
		m_TriggerLevel[n]=0;
	}
}

MatrixPlugin::~MatrixPlugin()
{
}

PluginInfo &MatrixPlugin::Initialise(const HostInfo *Host)
{	
	PluginInfo& Info = SpiralPlugin::Initialise(Host);
	Sample *NewSample = new Sample(Host->BUFSIZE);
	m_Output[0]=NewSample;
	NewSample = new Sample(Host->BUFSIZE);
	m_Output[1]=NewSample;	
	
	for (int n=0; n<16; n++)
	{
		NewSample = new Sample(Host->BUFSIZE);
		m_Output[n+2]=NewSample;	
	}
	
	m_TickTime = 1.0f/(float)m_HostInfo->SAMPLERATE;
	
	return Info;
}

Fl_Window *MatrixPlugin::CreateGUI()
{
	m_GUI = new MatrixPluginGUI(m_PluginInfo.Width,
								  	    m_PluginInfo.Height,
										this,m_HostInfo);
	m_GUI->hide();
	return m_GUI;
}

void MatrixPlugin::Execute()
{	
	float Speed;

	for (int n=0; n<m_HostInfo->BUFSIZE; n++)
	{
		if (m_Input[1]) m_StepTime = GetInputCV(1,n);
		
		// inc time 
		m_Time+=m_TickTime;
		
		m_Output[0]->Set(n,m_CurrentNoteCV);
		m_Output[1]->Set(n,m_CurrentTriggerCV);

		if ((unsigned short)(GetInputVal(0,n)+MAX_LEV)/FREQ_CONV>=TRIG_THRESH) 
		{
			if (!m_Triggered)
			{
				float Freq=(unsigned short)(GetInputVal(0,n)+MAX_LEV)/FREQ_CONV;
				
				// Notes 0 to 16 trigger patterns 0 to 16
				// No other notes catered for
				for (int i=0; i<NUM_PATTERNS; i++)
				{
					if (Freq>NoteTable[i] && Freq<=NoteTable[i+1])
					{
						m_Current=i;
					}
				}
				
				// make it so the next note to trigger
				// will be the first one
				
				if (m_GUI) ((MatrixPluginGUI*)m_GUI)->UpdateValues();
				
//				m_Time=0;
//				m_Step=0;
				
				m_Time=m_StepTime*(1/m_Matrix[m_Current].Speed);
				m_Step=-1;

				m_Triggered=true;
			}
		}
		else
		{
			m_Triggered=false;
		}


		// set the individual triggers
		for (int t=0; t<NUM_PATTERNS; t++) m_Output[t+2]->Set(n,m_TriggerLevel[t]);

		if (m_Time>=m_StepTime*(1/m_Matrix[m_Current].Speed)) 
		{		
			m_Time=0;
			m_Step++;
			
			if (m_Step>=m_Matrix[m_Current].Length)
			{
				m_Step=0;
			}
			
			if (m_GUI) ((MatrixPluginGUI*)m_GUI)->SetLED(m_Step);
			
			// Reset the values
			m_CurrentTriggerCV=0;
			if (m_NoteCut) m_CurrentNoteCV=0;
			for (int t=0; t<NUM_PATTERNS; t++) 
			{
				m_Output[t+2]->Set(n,0);
				m_TriggerLevel[t]=0;
			}
			
			
			// Scan the matrix at current time 
			for (int i=0; i<MATY; i++)
			{
				if (m_Matrix[m_Current].Matrix[m_Step][i])
				{
					m_CurrentNoteCV=(short)(NoteTable[i+m_Matrix[m_Current].Octave*12]*FREQ_CONV)-MAX_LEV;
					m_CurrentTriggerCV=MAX_LEV;
					m_TriggerLevel[i]=MAX_LEV;
				}
			}
			
			// reset the triggers between steps to clear them
			// otherwise consecutive events wont get triggered
			m_Output[1]->Set(n,0);
			for (int t=0; t<NUM_PATTERNS; t++) m_Output[t+2]->Set(n,0);
		}		
	}

 	Fl::check();
}

void MatrixPlugin::StreamOut(ostream &s)
{
	s<<m_Version<<" ";
	s<<m_Current<<" "<<m_Time<<" "<<m_Step<<" "<<m_Loop<<" "<<m_NoteCut<<" "<<endl;
	
	for (int n=0; n<NUM_PATTERNS; n++)	
	{
		s<<m_Matrix[n].Length<<" "<<m_Matrix[n].Speed<<" "<<m_Matrix[n].Octave<<endl;
		
		for (int y=0; y<MATY; y++)
		{
			for (int x=0; x<MATX; x++)
			{
				s<<m_Matrix[n].Matrix[x][y]<<" ";
			}
			s<<endl;
		}
		s<<endl;
	}
}

void MatrixPlugin::StreamIn(istream &s)
{
	int version;
	s>>version;
	
	s>>m_Current>>m_Time>>m_Step>>m_Loop>>m_NoteCut;
	
	for (int n=0; n<NUM_PATTERNS; n++)	
	{	
		s>>m_Matrix[n].Length>>m_Matrix[n].Speed>>m_Matrix[n].Octave;
		
		for (int y=0; y<MATY; y++)
		for (int x=0; x<MATX; x++)
		{
			s>>m_Matrix[n].Matrix[x][y];
		}
	}
}


int main()
{
	Fl::visual(FL_DOUBLE|FL_RGB);
	
	HostInfo host;
	host.BUFSIZE=256;
	host.SAMPLERATE=44100;
	host.OUTPUTFILE="/dev/dsp";
	host.MIDIFILE="/dev/midi";
	host.POLY=1;
	host.GUI_COLOUR=100;
	
	
	SpiralPlugin* test = CreateInstance();
	test->Initialise(&host);
    test->CreateGUI();
		
    for (;;) 
	{
    	if (!Fl::check()) break;
		test->Execute();    	
  	}
	
	delete test;	
	
	return 1;	
}
