/*  SpiralSound
 *  Copyleft (C) 2001 David Griffiths <dave@pawfal.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/ 

#include <vector>
#include <string>
#include <iostream>
#include <FL/Fl_Window.h>
#include "../Sample.h"
#include <math.h>

#ifndef SPIRALPLUGIN
#define SPIRALPLUGIN

struct PluginInfo
{
	string Name;
	int Width;
	int Height;
	int NumInputs;
	int NumOutputs;
	vector<string> PortTips;
	char BitMap[40][40][3]; 
};

struct HostInfo
{
	int    BUFSIZE;	
	int    FRAGSIZE;  
	int    FRAGCOUNT; 	
	int    SAMPLERATE;
	string OUTPUTFILE;
	string MIDIFILE;
	int    POLY;
	int    GUI_COLOUR;
};

class SpiralPlugin
{
public:
	SpiralPlugin();
	virtual ~SpiralPlugin();
		
	virtual PluginInfo& Initialise(const HostInfo *Host);
	virtual void        Execute()=0;
	virtual Fl_Window*  CreateGUI()=0;
	virtual void	    StreamOut(ostream &s)=0;
	virtual void	    StreamIn(istream &s)=0;

	const HostInfo*     GetHostInfo() { return m_HostInfo; }
	bool                GetOutput(unsigned int n, Sample **s);
	bool                SetInput(unsigned int n, const Sample *s);
	const Sample*       GetInput(unsigned int n) { return m_Input[n]; }
	string 				GetName() { return m_PluginInfo.Name; }
	
	bool ValueChanged();
	void SetValueChanged(bool s) { m_ValuesChanged=s; }

	void UpdatePluginInfoWithHost() { if (UpdateInfo!=NULL) UpdateInfo(m_HostID,(void*)&m_PluginInfo); }
	void SetUpdateInfoCallback(int ID, void(*s)(int, void *)) { m_HostID=ID; UpdateInfo=s; }
	
protected:

	short int GetInputVal(int n,int p) { if (m_Input[n]) return (*m_Input[n])[p]; else return 0; }
	float     GetInputCV(int n,int p)  { if (m_Input[n]) return (*m_Input[n])[p]*VAL2CV; else return 0.0; }
	float     GetInputUCV(int n,int p) { if (m_Input[n]) return (*m_Input[n])[p]*UVAL2CV; else return 0.0; }
	
	Fl_Window 	   *m_GUI;
	const HostInfo *m_HostInfo;
	PluginInfo      m_PluginInfo;
	bool 			m_ValuesChanged;
	int             m_Version;		
	
	vector<const Sample*> m_Input;
	vector<Sample*> m_Output;

private:

	void (*UpdateInfo)(int n,void *);	
	int m_HostID;
};

#endif
