/*  SpiralSound
 *  Copyleft (C) 2001 David Griffiths <dave@pawfal.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/ 
#include "TestPlugin.h"
#include "TestPluginGUI.h"
#include <FL/Fl_Button.h>
#include "SpiralIcon.xpm"

SpiralPlugin* CreateInstance()
{
	return new TestPlugin;
}

Fl_Pixmap* GetIcon()
{
	return new Fl_Pixmap(SpiralIcon_xpm);
}

int GetID()
{
	return 0x0016;
}

///////////////////////////////////////////////////////

TestPlugin::TestPlugin()
{
	m_PluginInfo.Name="Test";
	m_PluginInfo.Width=220;
	m_PluginInfo.Height=125;
	m_PluginInfo.NumInputs=1;
	m_PluginInfo.NumOutputs=2;
	m_PluginInfo.PortTips.push_back("Input");	
	m_PluginInfo.PortTips.push_back("Out one");
	m_PluginInfo.PortTips.push_back("Out two");
}

TestPlugin::~TestPlugin()
{
}

PluginInfo &TestPlugin::Initialise(const HostInfo *Host)
{	
	PluginInfo& Info = SpiralPlugin::Initialise(Host);
	Sample *NewSample = new Sample(Host->BUFSIZE);
	m_Output[0]=NewSample;	
	NewSample = new Sample(Host->BUFSIZE);
	m_Output[1]=NewSample;
	return Info;
}

Fl_Window *TestPlugin::CreateGUI()
{
	m_GUI = new TestPluginGUI(m_PluginInfo.Width,
										  m_PluginInfo.Height,
										  this,m_HostInfo);									  
  	m_GUI->hide();
	return m_GUI;
}

void TestPlugin::Execute()
{
	// Just copy the data through.
	if (m_Output[0]) m_Output[0]->Zero();
	if (m_Input[0])  m_Output[0]->Mix(*m_Input[0],0);
	if (m_Output[1]) m_Output[1]->Zero();
	if (m_Input[0])  m_Output[1]->Mix(*m_Input[0],0);
	
 	Fl::check();
}
 
void TestPlugin::ButtonPress()
{
	m_PluginInfo.Name="Test";
	m_PluginInfo.Width=220;
	m_PluginInfo.Height=125;
	m_PluginInfo.NumInputs=2;
	m_PluginInfo.NumOutputs=2;
	m_PluginInfo.PortTips.clear();
	m_PluginInfo.PortTips.push_back("Input");	
	m_PluginInfo.PortTips.push_back("Input2");
	m_PluginInfo.PortTips.push_back("Out one");
	m_PluginInfo.PortTips.push_back("Out two");

	UpdatePluginInfoWithHost();
}
	 
int main()
{
	Fl::visual(FL_DOUBLE|FL_RGB);
	
	HostInfo host;
	host.BUFSIZE=256;
	host.SAMPLERATE=44100;
	host.OUTPUTFILE="/dev/dsp";
	host.MIDIFILE="/dev/midi";
	host.POLY=1;
	host.GUI_COLOUR=100;
	
	
	SpiralPlugin* test = CreateInstance();
	test->Initialise(&host);
    test->CreateGUI();
		
    for (;;) 
	{
    	if (!Fl::check()) break;
		test->Execute();    	
  	}
	
	delete test;	
	
	return 1;	
}
