/*  SpiralSynth
 *  Copyleft (C) 2000 David Griffiths <dave@pawfal.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/ 

#include <iostream.h>
#include <fstream>
#include <stdio.h>
#include <stdlib.h>
#include "RiffWav.h"
#include "SpiralInfo.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

const int HEADERLEN = (4+24+8);

#if __BYTE_ORDER == BIG_ENDIAN
#define SWAPINT(a) (a)=(((a)&0x000000ff)<<24)|(((a)&0x0000ff00)<<8)|\
						    (((a)&0x00ff0000)>>8)|(((a)&0xff000000)>>24)
#endif

int WavFile::Open(string FileName, Mode mode, Channels channels=MONO)
{
	if (m_Stream!=NULL) 
	{
		cerr<<"WavFile: File already open ["<<FileName<<"]"<<endl;
		return 0;
	}
	
	if (mode==WRITE) m_Stream = fopen (FileName.c_str(), "wb"); 
	else m_Stream = fopen (FileName.c_str(), "rb"); 
	
	if (m_Stream == (FILE*)0)
	{
		cerr<<"WavFile: File open error ["<<FileName<<"]"<<endl;
		return 0;
	}

	//Write the header

	if (mode==WRITE)
	{		
		m_Header.RiffName[0]='R';
		m_Header.RiffName[1]='I';
		m_Header.RiffName[2]='F';
		m_Header.RiffName[3]='F';
	
		m_Header.RiffFileLength=HEADERLEN; // bzzt - wrong
	
		m_Header.RiffTypeName[0]='W';
		m_Header.RiffTypeName[1]='A';
		m_Header.RiffTypeName[2]='V';
		m_Header.RiffTypeName[3]='E';
			
		m_Header.FmtName[0]='f';
		m_Header.FmtName[1]='m';
		m_Header.FmtName[2]='t';
		m_Header.FmtName[3]=' ';
		
		m_Header.FmtLength=0x00000010; // length of fmt data (16 bytes)
		m_Header.FmtTag=0x0001;        // Format tag: 1 = PCM
		if (channels==STEREO) m_Header.FmtChannels=2;     
		else m_Header.FmtChannels=1;     
		m_Header.FmtSamplerate=WavFile::m_Samplerate;
	
		m_Header.FmtBitsPerSample=16;
		m_Header.FmtBytesPerSec=m_Header.FmtSamplerate*m_Header.FmtBlockAlign;
		m_Header.FmtBlockAlign=m_Header.FmtChannels*m_Header.FmtBitsPerSample/8;	

		m_Header.DataName[0]='d';	
		m_Header.DataName[1]='a';
		m_Header.DataName[2]='t';
		m_Header.DataName[3]='a';
		
		m_Header.DataLengthBytes=0;
			
		#if __BYTE_ORDER == BIG_ENDIAN
		SWAPINT(m_Header.RiffFileLength);
		SWAPINT(m_Header.FmtSamplerate);
		SWAPINT(m_Header.FmtBytesPerSec);
		SWAPINT(m_Header.DataLengthBytes);
		#endif
			
		fwrite(&m_Header,1,sizeof(CanonicalWavHeader),m_Stream);		
	
		return 1;
	}
	else
	if (mode==READ)
	{
		fread(&m_Header,sizeof(CanonicalWavHeader),1,m_Stream);
		
		/*cerr<<m_Header.RiffName<<endl;
		cerr<<m_Header.RiffFileLength<<endl;
		cerr<<m_Header.RiffTypeName<<endl;
		cerr<<m_Header.FmtName<<endl;
		cerr<<m_Header.FmtLength<<endl;
		cerr<<m_Header.FmtTag<<endl;
		cerr<<m_Header.FmtChannels<<endl;
		cerr<<m_Header.FmtSamplerate<<endl;
		cerr<<m_Header.FmtBytesPerSec<<endl;
		cerr<<m_Header.FmtBlockAlign<<endl;
		cerr<<m_Header.FmtBitsPerSample<<endl;
		cerr<<m_Header.DataName<<endl;
		cerr<<m_Header.DataLengthBytes<<endl;*/
				
		// check we have a wav file here
		if (m_Header.RiffName[0]=='R' &&
			m_Header.RiffName[1]=='I' &&
			m_Header.RiffName[2]=='F' &&
			m_Header.RiffName[3]=='F')
		{
			return 1;
		}
		
		fclose(m_Stream);
		cerr<<"WavFile: File open error, wrong format ["<<FileName<<"]"<<endl;
		return 0;
	}
}

int WavFile::Close()
{
	if (m_Stream==NULL) 
	{
		return 0;
	}

	// write the total length in	
	fseek(m_Stream, 40, SEEK_SET);
	fwrite(&m_Header.DataLengthBytes,4,1,m_Stream);
	fclose(m_Stream); 

	m_Stream=NULL;
	return 1;	
}

int WavFile::Save(Sample &data)
{
	if (m_Stream==NULL || data.GetLength()==0)
	{
		return 0;
	}
	
	m_Header.DataLengthBytes+=data.GetLength()*2;
	fwrite(data.GetBuffer(),sizeof(data.GetBuffer()),data.GetLength()/2,m_Stream);
	return 1;
}

int WavFile::GetSize()
{	
	if (m_Header.FmtBitsPerSample!=8 && m_Header.FmtBitsPerSample!=16)
	{
		cerr<<"WavFile Warning: FmtBitsPerSample="<<m_Header.FmtBitsPerSample<<" (can't cope, treating as 16)"<<endl;
		m_Header.FmtBitsPerSample=16;
	}
	
	if (m_Header.FmtBitsPerSample==8) return m_Header.DataLengthBytes;
	else return m_Header.DataLengthBytes/2;	
}

int WavFile::Load(Sample &data)
{
	if (m_Header.FmtChannels>1) // mix the channels into a mono buffer 
	{
		cerr<<"WavFile::Load - Channels = "<<m_Header.FmtChannels<<
		" Mixing down to mono..."<<endl;

		short *TempBuf = new short[GetSize()];
		if (m_Header.DataLengthBytes!=(int)fread(TempBuf,1,m_Header.DataLengthBytes,m_Stream))
    	{
    	    cerr<<"WavFile: Read error"<<endl;
    		return 0; 
    	}
		
		for (int n=0; n<GetSize()/m_Header.FmtChannels; n++)
		{
			long value=0;
			for (int i=0; i<m_Header.FmtChannels; i++)
			{
				value+=TempBuf[(n*m_Header.FmtChannels)+i];
			}
			value/=m_Header.FmtChannels;
			
			data.Set(n,value);
		}
	}
	else // we can read the data directly in, it's mono.
	{
    	if (m_Header.DataLengthBytes==
		(int)fread(data.GetNonConstBuffer(),1,m_Header.DataLengthBytes,m_Stream))
    	{
    	    return 1;
    	}
	
    	cerr<<"WavFile: Read error"<<endl;
    	return 0;
	}
	
	return 0;
}
