/*  SpiralSynthModular
 *  Copyleft (C) 2001 David Griffiths <dave@pawfal.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/ 

#include <string>
#include <iostream>
#include <fstream>
#include <string.h>
#include <stdio.h>
#include <FL/Fl.H>
#include <FL/Enumerations.H>
#include <FL/fl_file_chooser.h>
#include <FL/fl_ask.h>
#include <FL/Fl_Box.h>

#include "SpiralSynthModular.h"
#include "SpiralSound/PluginManager.h"
//#include "SSM.xpm"

const static string TITLEBAR = "SpiralSynthModular 0.0.6";

static const int FILE_VERSION = 1;
static int Numbers[512];

static const int MAIN_WIDTH     = 724;
static const int MAIN_HEIGHT    = 420;
static const int SLIDER_WIDTH   = 15;
static const int TOOLBOX_HEIGHT = 330;
static const int TOOLBOX_WIDTH  = 132+SLIDER_WIDTH;
static const int ICON_DEPTH     = 3;
static const int COMMENT_ID     = -1;

map<int,DeviceWin*> SynthModular::m_DeviceWinMap;

DeviceWin::~DeviceWin()
{	
} 
		
SynthModular::SynthModular():
m_NextID(0),
m_NextPluginButton(1),
m_NextPluginButtonXPos(5),
m_NextPluginButtonYPos(20)
{ 	
	m_Info.BUFSIZE    = SpiralInfo::BUFSIZE;
	m_Info.FRAGSIZE   = SpiralInfo::FRAGSIZE;
	m_Info.FRAGCOUNT  = SpiralInfo::FRAGCOUNT;	
	m_Info.SAMPLERATE = SpiralInfo::SAMPLERATE;
	m_Info.OUTPUTFILE = SpiralInfo::OUTPUTFILE;
	m_Info.MIDIFILE   = SpiralInfo::MIDIFILE; 
	m_Info.POLY       = SpiralInfo::POLY;
	//m_Info.GUI_COLOUR = SpiralInfo::GUI_COLOUR;
	
	for (int n=0; n<512; n++) Numbers[n]=n;	
}

SynthModular::~SynthModular()
{ 
	ClearUp();
	PluginManager::Get()->PackUpAndGoHome();
}

void SynthModular::ClearUp()
{
	m_Canvas->Clear();

	for(map<int,DeviceWin*>::iterator i=m_DeviceWinMap.begin();
		i!=m_DeviceWinMap.end(); i++)
	{
		delete i->second->m_DeviceGUI;
		delete i->second->m_Device;
	}
	
	m_DeviceWinMap.clear();
	m_NextID=0;
}

Fl_Window *SynthModular::CreateWindow()
{
	int xoff=0, yoff=0;

	m_Window = new Fl_Window(MAIN_WIDTH, MAIN_HEIGHT, TITLEBAR.c_str());
	m_Window->color(SpiralSynthModularInfo::GUICOL_Tool);
 	
	m_ToolBox = new Fl_Scroll(0,0,TOOLBOX_WIDTH, TOOLBOX_HEIGHT, "");
    m_ToolBox->type(Fl_Scroll::VERTICAL_ALWAYS);
	m_ToolBox->box(FL_UP_BOX);
    m_ToolBox->labeltype(FL_ENGRAVED_LABEL);
    m_ToolBox->align(FL_ALIGN_TOP_LEFT|FL_ALIGN_INSIDE);
	m_ToolBox->color(SpiralSynthModularInfo::GUICOL_Tool);
	m_ToolBox->user_data((void*)(this));
	m_Window->add(m_ToolBox);  

	Fl_Box *Label = new Fl_Box(5,3,100,20,"ToolBox");
	Label->box(FL_FLAT_BOX);
	Label->color(SpiralSynthModularInfo::GUICOL_Tool);
	Label->labeltype(FL_ENGRAVED_LABEL);
	Label->align(FL_ALIGN_TOP_LEFT|FL_ALIGN_INSIDE);
	m_ToolBox->add(Label);
	
	m_ToolPack = new Fl_Pack(5,20,TOOLBOX_WIDTH-10, TOOLBOX_HEIGHT-40,"");
    m_ToolPack->type(FL_VERTICAL);
	m_ToolPack->box(FL_NO_BOX);
	m_ToolPack->color(SpiralSynthModularInfo::GUICOL_Tool);
    m_ToolPack->user_data((void*)(this));
	m_ToolBox->add(m_ToolPack);  
	
	xoff=0; yoff=TOOLBOX_HEIGHT;
	m_Buttons = new Fl_Group(xoff, yoff, TOOLBOX_WIDTH, MAIN_HEIGHT-TOOLBOX_HEIGHT, "");
    m_Buttons->type(1);
	m_Buttons->color(SpiralSynthModularInfo::GUICOL_Tool);
	m_Buttons->box(FL_UP_BOX);
	m_Buttons->user_data((void*)(this));
	m_Window->add(m_Buttons); 

	m_Load = new Fl_Button(5+xoff, 5+yoff, TOOLBOX_WIDTH-10, 20, "Load");
	m_Load->labelsize(10);
	m_Load->color(SpiralSynthModularInfo::GUICOL_Button);
    m_Load->type(0);		 
	m_Load->callback((Fl_Callback*)cb_Load);
	
	m_Save = new Fl_Button(5+xoff, 25+yoff, TOOLBOX_WIDTH-10, 20, "Save");
	m_Save->labelsize(10);
	m_Save->color(SpiralSynthModularInfo::GUICOL_Button);
    m_Save->type(0);		 
	m_Save->callback((Fl_Callback*)cb_Save);
	
	m_New = new Fl_Button(5+xoff, 45+yoff, TOOLBOX_WIDTH-10, 20, "New");
	m_New->labelsize(10);
	m_New->color(SpiralSynthModularInfo::GUICOL_Button);
    m_New->type(0);		 
	m_New->callback((Fl_Callback*)cb_New);

	m_NewComment = new Fl_Button(5+xoff, 65+yoff, TOOLBOX_WIDTH-10, 20, "Comment");
	m_NewComment->labelsize(10);
	m_NewComment->color(SpiralSynthModularInfo::GUICOL_Button);
    m_NewComment->type(0);		 
	m_NewComment->callback((Fl_Callback*)cb_NewComment);
		
	m_Status = new Fl_Output(TOOLBOX_WIDTH,MAIN_HEIGHT-20,724-TOOLBOX_WIDTH,20);
	m_Window->add(m_Status);   

	m_CanvasScroll = new Fl_Scroll(TOOLBOX_WIDTH, 0, 724-TOOLBOX_WIDTH, MAIN_HEIGHT-20, "");
	m_Window->add(m_CanvasScroll);
	m_Window->resizable(m_CanvasScroll);
	
	m_Canvas = new Fl_Canvas(-5000, -5000, 10000, 10000, "Canvas");
    m_Canvas->type(1);
	m_Canvas->box(FL_UP_BOX);
    m_Canvas->labeltype(FL_ENGRAVED_LABEL);
    m_Canvas->align(FL_ALIGN_TOP_LEFT|FL_ALIGN_INSIDE);
	m_Canvas->color(SpiralSynthModularInfo::GUICOL_Canvas);
	m_Canvas->user_data((void*)(this));
	m_Canvas->SetConnectionCallback((Fl_Callback*)cb_Connection);
	m_Canvas->SetUnconnectCallback((Fl_Callback*)cb_Unconnect);
	m_Canvas->SetStatusBar(m_Status);
	m_CanvasScroll->add(m_Canvas);   
	
	return m_Window;
}

void SynthModular::LoadPlugins()
{

	int Width  = 40;
	int Height = 40;	

	//Fl_Window* Splash = new Fl_Window(m_Window->x()+((m_Window->w()/2)-170),
	//								  m_Window->y()+((m_Window->h()/2)-100),
	//								  320,200,"SSM");
	//Splash->border(0);
	//Fl_Button* pbut = new Fl_Button(0,0,320,200,"");
	//pbut->box(FL_NO_BOX);
	//Fl_Pixmap pic(SSM_xpm);
	//pic.label(pbut);
	//Splash->add(pbut);
	//Splash->show();

	//Fl::check();

	char message[512];
	int ID=-1;
	int Icon=0;
	Fl_Pack *IconPack;

	IconPack = new Fl_Pack(0,0,TOOLBOX_WIDTH-SLIDER_WIDTH,Height,"");
    IconPack->type(FL_HORIZONTAL);
	IconPack->color(SpiralSynthModularInfo::GUICOL_Tool);
	IconPack->user_data((void*)(this));  
	m_ToolPack->add(IconPack);

	for (vector<string>::iterator i=SpiralSynthModularInfo::PLUGINVEC.begin();
		 i!=SpiralSynthModularInfo::PLUGINVEC.end(); i++)
	{
		string Fullpath=SpiralSynthModularInfo::PLUGIN_PATH+*i;

		ID=PluginManager::Get()->LoadPlugin(Fullpath.c_str());
	
		if (Icon>=ICON_DEPTH)
		{
			Icon=0;
			IconPack = new Fl_Pack(0,0,TOOLBOX_WIDTH-SLIDER_WIDTH,Height,"");
    		IconPack->type(FL_HORIZONTAL);
			IconPack->color(SpiralSynthModularInfo::GUICOL_Tool);
			IconPack->user_data((void*)(this));
			m_ToolPack->add(IconPack); 
		}
	
		Fl_Button *NewButton = new Fl_Button(0,0,Width,Height,"");
		NewButton->labelsize(10);
		Fl_Pixmap *Pix = PluginManager::Get()->GetPlugin(ID)->GetIcon();
		Pix->label(NewButton);
		IconPack->add(NewButton);
		NewButton->type(0);	
		NewButton->color(SpiralSynthModularInfo::GUICOL_Button);	 
		NewButton->callback((Fl_Callback*)cb_NewDevice,&Numbers[ID]);			
		NewButton->show();
		m_DeviceVec.push_back(NewButton);			
		Icon++;
		
		m_ToolBox->redraw();
		
		m_NextPluginButton++;
		
		sprintf(message,"Loaded plugin: %s",i->c_str());
		m_Status->value(message);
		Fl::check();
	}
	
	//Splash->hide();
	//delete Splash;
}

void SynthModular::DoIdle()
{
	// see if any need deleting		
	for (map<int,DeviceWin*>::iterator i=m_DeviceWinMap.begin(); 
		 i!=m_DeviceWinMap.end(); i++)
	{	
		if (i->second->m_DeviceGUI->Killed())
		{
			m_Canvas->RemoveDevice(i->second->m_DeviceGUI);
			i->second->m_DeviceGUI->Clear();
			delete i->second->m_DeviceGUI;			
			if (i->second->m_Device) delete i->second->m_Device;
			m_DeviceWinMap.erase(i);
		}
	}
		
	// run the plugins
	for (map<int,DeviceWin*>::iterator i=m_DeviceWinMap.begin(); 
		 i!=m_DeviceWinMap.end(); i++)
	{	
		if (i->second->m_Device) i->second->m_Device->Execute();
	}
	
	
	// Check buttons for mouseover
	// Bit of a cludge this, both where it is, and 
	// implementation - but it works surprisingly!
	int n=0;
	for(vector<Fl_Button*>::iterator i=m_DeviceVec.begin();
		i!=m_DeviceVec.end(); i++)
	{
		if (Fl::belowmouse()==*i)
		{
			m_Canvas->DisplayText(SpiralSynthModularInfo::PLUGINVEC[n]);
		}		
		n++;
	}
}

DeviceGUIInfo SynthModular::BuildDeviceGUIInfo(PluginInfo &PInfo)
{
	DeviceGUIInfo Info;
	int Height=50;
	
	// tweak the size if we have too many ins/outs
	if (PInfo.NumInputs>4 || PInfo.NumOutputs>4)
	{
		if (PInfo.NumInputs<PInfo.NumOutputs)
		{
			Height=PInfo.NumOutputs*10+5;
		}
		else
		{
			Height=PInfo.NumInputs*10+5;
		}
	}
	
	// Make the guiinfo struct
	Info.XPos       = 0;
	Info.YPos       = 0;
	Info.Width      = 40;
	Info.Height     = Height;
	Info.NumInputs  = PInfo.NumInputs;
	Info.NumOutputs = PInfo.NumOutputs;
	Info.Name       = PInfo.Name;
	Info.PortTips   = PInfo.PortTips;
	
	return Info;
}

DeviceWin* SynthModular::NewDeviceWin(int n, int x=TOOLBOX_WIDTH+50, int y=100)
{
	DeviceWin *nlw = new DeviceWin;
	
	nlw->m_Device=PluginManager::Get()->GetPlugin(n)->CreateInstance();

	if (!nlw->m_Device)
	{
		return NULL;
	}
	
	PluginInfo PInfo = nlw->m_Device->Initialise(&m_Info);
	Fl_Window *temp  = nlw->m_Device->CreateGUI();
	
	Fl_Pixmap *Pix   = PluginManager::Get()->GetPlugin(n)->GetIcon();
	nlw->m_PluginID  = n;
	
	DeviceGUIInfo Info=BuildDeviceGUIInfo(PInfo);
	Info.XPos       = x; //TOOLBOX_WIDTH+(rand()%400);
	Info.YPos       = y; //rand()%400;
	
	nlw->m_DeviceGUI = new Fl_DeviceGUI(Info, temp, Pix);
			
	m_Canvas->add(nlw->m_DeviceGUI);
	m_Canvas->redraw();
	return nlw;
}


void SynthModular::AddDevice(int n)
{
	//cerr<<"Adding "<<m_NextID<<endl;
	DeviceWin* temp = NewDeviceWin(n);
	if (temp)
	{
		int ID=m_NextID++;
		temp->m_DeviceGUI->SetID(ID);
		temp->m_Device->SetUpdateInfoCallback(ID,cb_UpdatePluginInfo);
		m_DeviceWinMap[ID]=temp;
	}
}

DeviceWin* SynthModular::NewComment(int n, int x=TOOLBOX_WIDTH+50, int y=100)
{	
	DeviceWin *nlw = new DeviceWin;
	
	nlw->m_Device=NULL;
	nlw->m_PluginID  = COMMENT_ID;
	
	DeviceGUIInfo Info;	
	
	Info.XPos       = x; 
	Info.YPos       = y; 
	Info.Width      = 50;
	Info.Height     = 20;
	Info.NumInputs  = 0;
	Info.NumOutputs = 0;
	Info.Name = "";
	
	nlw->m_DeviceGUI = new Fl_CommentGUI(Info, NULL, NULL);
			
	m_Canvas->add(nlw->m_DeviceGUI);
	m_Canvas->redraw();
	
	return nlw;
}

void SynthModular::AddComment(int n)
{
	//cerr<<"Adding "<<m_NextID<<endl;
	DeviceWin* temp = NewComment(n);
	if (temp)
	{
		int ID=m_NextID++;
		temp->m_DeviceGUI->SetID(ID);
		m_DeviceWinMap[ID]=temp;
	}
}

istream &operator>>(istream &s, SynthModular &o)
{	
	string dummy;
	int ver;
	s>>dummy>>dummy>>dummy>>ver;
	
	if (ver>FILE_VERSION)
	{
		cerr<<"Bad file, or more recent version."<<endl;
		return s;
	}
		
	int Num, ID, PluginID, x,y;
	s>>dummy>>Num;
	
	for(int n=0; n<Num; n++)
	{	
		//cerr<<"Loading Device "<<n<<endl;
		
		s>>dummy;
		s>>ID; 
		s>>dummy;
		s>>PluginID;
		s>>x>>y;
		
		// Check we're not duplicating an ID
		assert(o.m_DeviceWinMap.find(ID)==o.m_DeviceWinMap.end());
		
		if (PluginID==COMMENT_ID)
		{
			DeviceWin* temp = o.NewComment(PluginID, x, y);
			if (temp)
			{
				temp->m_DeviceGUI->SetID(ID);
				o.m_DeviceWinMap[ID]=temp;
				((Fl_CommentGUI*)(o.m_DeviceWinMap[ID]->m_DeviceGUI))->StreamIn(s); // load the plugin	
				if (o.m_NextID<=ID) o.m_NextID=ID+1;
			}
		}
		else
		{
			DeviceWin* temp = o.NewDeviceWin(PluginID, x, y);
			if (temp)
			{
				temp->m_DeviceGUI->SetID(ID);
				temp->m_Device->SetUpdateInfoCallback(ID,o.cb_UpdatePluginInfo);
				o.m_DeviceWinMap[ID]=temp;
				o.m_DeviceWinMap[ID]->m_Device->StreamIn(s); // load the plugin		
				if (o.m_NextID<=ID) o.m_NextID=ID+1;
			}
		}	
	}

	s>>*o.m_Canvas;

	return s;
}

ostream &operator<<(ostream &s, SynthModular &o)
{
	s<<"SpiralSynthModular File Ver 1"<<endl;
		
	// save out the SynthModular
	s<<"SectionList"<<endl;
	s<<o.m_DeviceWinMap.size()<<endl;
	
	for(map<int,DeviceWin*>::iterator i=o.m_DeviceWinMap.begin();
		i!=o.m_DeviceWinMap.end(); i++)
	{		
		s<<endl;
		s<<"Device ";
		s<<i->first<<" "; // save the id
		s<<"Plugin ";
		s<<i->second->m_PluginID<<endl;
		s<<i->second->m_DeviceGUI->x()<<" ";
		s<<i->second->m_DeviceGUI->y();
		s<<endl;
		
		if (i->second->m_PluginID==COMMENT_ID)
		{
			// save the comment gui
			((Fl_CommentGUI*)(i->second->m_DeviceGUI))->StreamOut(s);
		}
		else
		{		
			// save the plugin
			i->second->m_Device->StreamOut(s); 
		}
		s<<endl;
	}
	
	s<<endl<<*o.m_Canvas<<endl;
	
	return s;
}

inline void SynthModular::cb_Load_i(Fl_Button* o, void* v)
{
	if (m_DeviceWinMap.size()>0 && !fl_ask("Load - Loose changes to current design?"))
	{
		return;
	}	
	
	char *fn=fl_file_chooser("Load a design", "*.ssm", NULL);
	
	if (fn && fn!='\0')
	{
		ifstream inf(fn);
		
		if (inf)
		{
			ClearUp();
			inf>>*this;
			
			string label=TITLEBAR+" "+fn;
			m_Window->label(label.c_str());
		}
	}
}

void SynthModular::cb_Load(Fl_Button* o, void* v)
{((SynthModular*)(o->parent()->user_data()))->cb_Load_i(o,v);}

inline void SynthModular::cb_Save_i(Fl_Button* o, void* v)
{
	char *fn=fl_file_chooser("Save a design", "*.ssm", NULL);
		
	if (fn && fn!='\0')
	{
		ifstream ifl(fn);
		if (ifl)
		{
			if (!fl_ask("File [%s] exists, overwrite?",fn))
			{
				return;
			}			
		}
	
		ofstream of(fn);
		
		if (of)
		{			
			of<<*this;
			
			string label=TITLEBAR+" "+fn;
			m_Window->label(label.c_str());
		}
	}
}
void SynthModular::cb_Save(Fl_Button* o, void* v)
{((SynthModular*)(o->parent()->user_data()))->cb_Save_i(o,v);}

inline void SynthModular::cb_New_i(Fl_Button* o, void* v)
{
	if (m_DeviceWinMap.size()>0 && !fl_ask("New - Loose changes to current design?"))
	{
		return;
	}	

	m_Window->label(TITLEBAR.c_str());

	ClearUp();
}
void SynthModular::cb_New(Fl_Button* o, void* v)
{((SynthModular*)(o->parent()->user_data()))->cb_New_i(o,v);}

inline void SynthModular::cb_NewDevice_i(Fl_Button* o, void* v)
{
	AddDevice(*((int*)v));
}
void SynthModular::cb_NewDevice(Fl_Button* o, void* v)
{((SynthModular*)(o->parent()->user_data()))->cb_NewDevice_i(o,v);}

inline void SynthModular::cb_NewComment_i(Fl_Button* o, void* v)
{
	AddComment(-1);
}
void SynthModular::cb_NewComment(Fl_Button* o, void* v)
{((SynthModular*)(o->parent()->user_data()))->cb_NewComment_i(o,v);}

inline void SynthModular::cb_Connection_i(Fl_Canvas* o, void* v)
{
	CanvasWire *Wire;
	Wire=(CanvasWire*)v;
			
	map<int,DeviceWin*>::iterator si=m_DeviceWinMap.find(Wire->OutputID);
	if (si==m_DeviceWinMap.end())
	{
		cerr<<"Can't find source device "<<Wire->OutputID<<endl;
		return;
	}
	
	map<int,DeviceWin*>::iterator di=m_DeviceWinMap.find(Wire->InputID);
	if (di==m_DeviceWinMap.end())
	{
		cerr<<"Can't find destination device "<<Wire->InputID<<endl;
		return;
	}

	Sample *sample=NULL;
	
	if (!si->second->m_Device->GetOutput(Wire->OutputPort,&sample))
	{ cerr<<"Can't find source device's Output"<<endl; return; }
	
	if (!di->second->m_Device->SetInput(Wire->InputPort,(const Sample*)sample))
	{ cerr<<"Can't find destination device's Input"<<endl; return;	}
	
}
void SynthModular::cb_Connection(Fl_Canvas* o, void* v)
{((SynthModular*)(o->user_data()))->cb_Connection_i(o,v);}


inline void SynthModular::cb_Unconnect_i(Fl_Canvas* o, void* v)
{
	CanvasWire *Wire;
	Wire=(CanvasWire*)v;
	
	//cerr<<Wire->InputID<<" "<<Wire->InputPort<<endl;
	
	map<int,DeviceWin*>::iterator di=m_DeviceWinMap.find(Wire->InputID);
	if (di==m_DeviceWinMap.end())
	{
		cerr<<"Can't find destination device "<<Wire->InputID<<endl;
		return;
	}

	if (!di->second->m_Device->SetInput(Wire->InputPort,NULL))
	{ cerr<<"Can't find destination device's Input"<<endl; return;	}
	
}
void SynthModular::cb_Unconnect(Fl_Canvas* o, void* v)
{((SynthModular*)(o->user_data()))->cb_Unconnect_i(o,v);}

//////////////////////////////////////////////////////////

void SynthModular::cb_UpdatePluginInfo(int ID, void *PInfo)
{
	map<int,DeviceWin*>::iterator i=m_DeviceWinMap.find(ID);
	if (i!=m_DeviceWinMap.end())
	{
		DeviceGUIInfo Info=BuildDeviceGUIInfo(*((PluginInfo*)PInfo));
	
		(*i).second->m_DeviceGUI->Setup(Info);
		(*i).second->m_DeviceGUI->redraw();
	}
}

//////////////////////////////////////////////////////////

#include <sys/resource.h> 

int main(int argc, char **argv)
{	
	srand(time(NULL));
	SpiralSynthModularInfo::Get()->LoadPrefs();
	
	//if (setpriority(PRIO_PROCESS,0,-20)!=-1)
	//{
	//	cerr<<"Set highest scheduling priority"<<endl;
	//}
	
	Fl::visual(FL_SINGLE|FL_RGB);
	
	SynthModular *synth=new SynthModular;
	synth->CreateWindow();
    synth->m_Window->show(argc, argv);
			
	synth->LoadPlugins();
	
    for (;;) 
	{
    	if (!Fl::check()) break;
		synth->DoIdle();    	
  	}
		
	delete synth;	
			
	return 1;
}

